#include "stdafx.h"

// This module implements the MFlatFile class.  Designed to
// provide access to fix-length columnar files

#include "sortdef.h"

IMPLEMENT_SERIAL( MFlatFile, CFile, 1 ) 

// CalcOffsets - Utitliy function, automatically calculates field
// offsets based on position and size
void MFlatFile::CalcOffsets()
{
	int i, Offset=0;
	for(i=0; i <GetFieldCount(); i++ )
	{
		GetField(i)->SetOffset( Offset );
		Offset += GetField(i)->GetSize();
	}
}

// Move - Moves an item from one location in an CobList to another location
void MFlatFile::Move( CObArray& List, int OldPos, int NewPos )
{
	CObject * Tmp;

	Tmp = List[OldPos];
	List.RemoveAt( OldPos, 1 );
	List.InsertAt( NewPos, Tmp );
}

// Reset - Clears out internal data for object.
void MFlatFile::Reset()
{
	int i;

	Close();
	FileName="";
	for( i=0; i < FldList.GetSize(); i++ )
		delete FldList[i];
	FldList.RemoveAll();

	RecordSize = 0;
}

// Close - Closes the data file, but keeps it's definition in memory
void MFlatFile::Close()
{
	if (m_hFile != (UINT)hFileNull)
		CFile::Close();	
}

// Returns an MField pointer, based on field name
MField* MFlatFile::GetField( const char * Fld )
{
	int i;
	for( i=0; i < GetFieldCount(); i++ )
		if( GetField(i)->IsName( Fld ) )
			return( GetField(i) );
	return( NULL );
}


// AddField - Adds a Field to a Flat File layout defintion
void MFlatFile::AddField( CString m_Name, CString m_Type, WORD m_Size, WORD m_Offset )
{
	InsertField( m_Name, m_Type, m_Size, m_Offset, GetFieldCount() );
}

// InsertField - Inserts a field anywhere in a Flat File layout definition
void MFlatFile::InsertField( CString m_Name, CString m_Type, WORD m_Size, WORD m_Offset, int i )
{
	MField * New;

	if( m_Type.CompareNoCase( "string" ) == 0 )
	{
		New = new F_String( m_Name, m_Size, m_Offset );
		FldList.InsertAt( i, New );
	}
	RecordSize += m_Size;
}

// SetField- Allows for the changing of a field type
void MFlatFile::SetField( CString m_Name, CString m_Type, WORD m_Size, WORD m_Offset,  int i )
{
	RemoveField( i );
	InsertField( m_Name, m_Type, m_Size, m_Offset, i );
}

// GetFieldDescr - Returns a description of a field
CString MFlatFile::GetFieldDescr( int i )
{
	CString Descr;

	if( i <0 || i> GetFieldCount() )
		Descr="";
	else
		Descr.Format( "%s, %s, %d, %d", GetFieldName(i), GetFieldType(i), GetFieldSize(i), GetFieldOffset(i) );
	return( Descr );
	
}

// Serialize - MFC support for File I/O
void MFlatFile::Serialize( CArchive & ar )
{
	CFile::Serialize(ar);

	if( ar.IsStoring() )
	 	ar << FileName;
	else
	{
		Close();
		ar >> FileName;
	}
	
	FldList.Serialize( ar );
	RecordSize=0;
	int i;
	for(i=0; i < GetFieldCount(); i++ )
		RecordSize += GetField(i)->GetSize();

}

// Open - Opens the Flat file, based on current filename
BOOL MFlatFile::Open( int Mode )
{
	BOOL IsOpen = CFile::Open( FileName, Mode );
	if( IsOpen )
	{
		Record = new char[RecordSize];
		if( Record )
			ReadRecord();
		if( (GetLength()/(DWORD)RecordSize) > (DWORD)0x0FFFF)
			ThrowFlatFileException( "Too many records", FileName );
		RecCount = (WORD)(GetLength()/(DWORD)RecordSize);
	}
	
	return( IsOpen );
}

// ReadRecord - Reads a record from a flat file
BOOL MFlatFile::ReadRecord()
{
	int i;

	Seek( RecNum * RecordSize, CFile::begin );
	Read( Record, RecordSize );

	for( i=0; i < GetFieldCount(); i++ )
		((MField*)FldList[i])->SetFromBuffer( Record );
	return(TRUE);
}

// WriteRecord - Writes a record to a flat file
BOOL MFlatFile::WriteRecord()
{
	int i;

	Seek( RecNum * RecordSize, CFile::begin );
	
	for( i=0; i < GetFieldCount(); i++ )
		((MField*)FldList[i])->GetToBuffer( Record );
	Write( Record, RecordSize );
	return(TRUE);
}


// FlatFileException - Handles errors generated by the Flat File class
IMPLEMENT_DYNAMIC( FlatFileException,CException )

FlatFileException::FlatFileException( const char *xMsg, const char *FileName, int Line, char * File )
{
	Msg.Format("%s %s (%s:%u)", xMsg, FileName, File, Line );
}
	
FlatFileException::FlatFileException( FlatFileException  & X )
{
	Msg = X.Msg;
}

BOOL FlatFileException::GetErrorMessage( LPTSTR lpszError, UINT nMaxError, PUINT pnHelpContext )
{
	if( pnHelpContext )
		*pnHelpContext=0;
	strncpy( lpszError, Msg, min( nMaxError, strlen(Msg)) );
	lpszError[min( nMaxError, strlen(Msg))]='\0';
	return( TRUE );

}


