////////////////////////////////////////////////////////////////////////
// The MFlatFile class contains and manages a collection of MField class
// objects (and it's derived classes), and associates them with an open
// file.
class MFlatFile : public CFile
{
public:
	DECLARE_SERIAL( MFlatFile )
	MFlatFile() { RecordSize=0; RecNum=0; Record=NULL; RecCount=(WORD)-1; }
	void Serialize( CArchive& ar );

	BOOL Open( int Mode );
	BOOL Goto( WORD xRecNum ) { if(xRecNum<RecCount) RecNum=xRecNum; return(xRecNum<RecCount);}
	BOOL ReadRecord();
	BOOL WriteRecord();
	virtual void Close();
	virtual void Reset();

//Field support
	void CalcOffsets();
	void Move( CObArray& List, int OldPos, int NewPos );
	void AddField( CString m_Name, CString m_Type, WORD m_Size, WORD m_Offset );
	void InsertField(CString m_Name, CString m_Type, WORD m_Size, WORD m_Offset,  int i );
	void SetField( CString m_Name, CString m_Type, WORD m_Size, WORD m_Offset,  int i );
	void RemoveField( int i ) { 	RecordSize -= GetFieldSize(i); delete GetField(i); FldList.RemoveAt(i); }

	// High-level functions
	int GetFieldCount() { return(FldList.GetSize()); }
	int GetRecSize() { return( RecordSize ); }
	WORD GetRecCount() {return( RecCount ); }
	CString GetFieldDescr( int i );
	void MoveFieldUp( int i ) { if( i ) Move( FldList, i, i-1 ); }
	void MoveFieldDown( int i ) { if( i < GetFieldCount() ) Move(FldList, i,i+1); }
	CString GetFieldAsString( int i );
	CString GetFieldAsString( const char * Fld );
	
	// Lowlevel functions
	MField* GetField(int i) { return(((MField*)FldList[i])); }
	MField* GetField( const char * Fld );
	CString GetFieldName( int i ) { return( GetField(i)->GetName());}
	CString GetFieldType( int i ) { return( GetField(i)->GetType());}
	WORD GetFieldSize( int i ) { return ( GetField(i)->GetSize()); }
	WORD GetFieldOffset( int i ) { return( GetField(i)->GetOffset() ); }

	void SetFile( const char * xFile ) { FileName = xFile; };
	const char *GetFile() { return( FileName ); }

	
protected:
	CObArray FldList;
	CString FileName;
	WORD RecordSize, RecNum, RecCount;
	char * Record;

};


////////////////////////////////////////////////////////////////////////
//  The FlatFileExcpetion class handles exceptions for the FlatFile class
#define ThrowFlatFileException(X,Y) throw new FlatFileException(X,Y,__LINE__,__FILE__)

class FlatFileException : public CException
{
public:
	DECLARE_DYNAMIC(FlatFileException)
public:
	FlatFileException( const char *Msg, const char *FileName, int Line, char * File );
	FlatFileException( FlatFileException  & X );
	virtual BOOL GetErrorMessage( LPTSTR lpszError, UINT nMaxError, PUINT pnHelpContext = NULL );
private:
	CString Msg;
};


