// SrtView.cpp : implementation of the CSrtView class
//

#include "stdafx.h"
#include "Srt.h"

#include "FldEdit.h"
#include "SrtEdit.h"
#include "OutEdit.h"
#include "sortdef.h"
#include "SrtDoc.h"
#include "SrtView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSrtView

IMPLEMENT_DYNCREATE(CSrtView, CFormView)

BEGIN_MESSAGE_MAP(CSrtView, CFormView)
	//{{AFX_MSG_MAP(CSrtView)
	ON_BN_CLICKED(IDC_FLDADD, OnFldadd)
	ON_BN_CLICKED(IDC_FLDDEL, OnFlddel)
	ON_BN_CLICKED(IDC_FLDUP, OnFldup)
	ON_BN_CLICKED(IDC_FLDDWN, OnFlddwn)
	ON_LBN_DBLCLK(IDC_FLDLIST, OnDblclkFldlist)
	ON_BN_CLICKED(IDC_SRTADD, OnSrtadd)
	ON_BN_CLICKED(IDC_SRTDEL, OnSrtdel)
	ON_BN_CLICKED(IDC_SRTDWN, OnSrtdwn)
	ON_BN_CLICKED(IDC_SRTUP, OnSrtup)
	ON_BN_CLICKED(IDC_OUTADD, OnOutadd)
	ON_BN_CLICKED(IDC_OUTDEL, OnOutdel)
	ON_BN_CLICKED(IDC_OUTDWN, OnOutdwn)
	ON_BN_CLICKED(IDC_OUTUP, OnOutup)
	ON_BN_CLICKED(IDC_PICKINP, OnPickinp)
	ON_BN_CLICKED(IDC_RUN, OnRun)
	ON_BN_CLICKED(IDC_PICKOUT, OnPickout)
	ON_EN_CHANGE(IDC_INPUTFILE, OnChangeFile)
	ON_LBN_DBLCLK(IDC_OUTLIST, OnDblclkOutlist)
	ON_LBN_DBLCLK(IDC_SRTLIST, OnDblclkSrtlist)
	ON_BN_CLICKED(IDC_CALCOFF, OnCalcOff)
	ON_BN_CLICKED(IDC_HASSEP, OnHassep)
	ON_BN_CLICKED(IDC_DESCEND, OnDescend)
	ON_EN_CHANGE(IDC_OUTPUTFILE, OnChangeFile)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSrtView construction/destruction

CSrtView::CSrtView()
	: CFormView(CSrtView::IDD)
{
	//{{AFX_DATA_INIT(CSrtView)
	m_InputFile = _T("");
	m_OutputFile = _T("");
	m_HasSep = FALSE;
	m_Descending = FALSE;
	//}}AFX_DATA_INIT
	// TODO: add construction code here

}

CSrtView::~CSrtView()
{
}

void CSrtView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSrtView)
	DDX_Control(pDX, IDC_KEYSIZE, m_KeySize);
	DDX_Control(pDX, IDC_STATUS, m_Status);
	DDX_Control(pDX, IDC_OUTLIST, m_OutList);
	DDX_Control(pDX, IDC_SRTLIST, m_SortList);
	DDX_Control(pDX, IDC_RECSIZE, m_RecSize);
	DDX_Control(pDX, IDC_FLDLIST, m_FldList);
	DDX_Text(pDX, IDC_INPUTFILE, m_InputFile);
	DDX_Text(pDX, IDC_OUTPUTFILE, m_OutputFile);
	DDX_Check(pDX, IDC_HASSEP, m_HasSep);
	DDX_Check(pDX, IDC_DESCEND, m_Descending);
	//}}AFX_DATA_MAP
}

BOOL CSrtView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CSrtView diagnostics

#ifdef _DEBUG
void CSrtView::AssertValid() const
{
	CFormView::AssertValid();
}

void CSrtView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CSrtDoc* CSrtView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSrtDoc)));
	return (CSrtDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSrtView message handlers

// OnUpdate - Updates dialog box with initial values
// (called on each document load for an SDI)
void CSrtView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	//CFormView::OnUpdate( pSender, lHint, pHint );

	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	int i;

	
	m_OutputFile = pDoc->Def.GetOutputFile();
	m_InputFile = pDoc->Def.GetFile();
	m_Descending = !pDoc->Def.GetAscending();
	m_HasSep= pDoc->Def.HasRecSep();
	
	if( m_FldList.GetCount() )
		m_FldList.ResetContent();

	if( m_SortList.GetCount() )
		m_SortList.ResetContent();
	if( m_OutList.GetCount() )
		m_OutList.ResetContent();


	for( i=0; i < pDoc->Def.GetFieldCount(); i++ )
		m_FldList.AddString( pDoc->Def.GetFieldDescr(i) );

	for( i=0; i < pDoc->Def.GetSortCount(); i++ )
		m_SortList.AddString( pDoc->Def.GetSortDescr(i) );

	for( i=0; i < pDoc->Def.GetOutputCount(); i++ )
		m_OutList.AddString( pDoc->Def.GetOutputDescr(i) );

	UpdateRecSize( pDoc->Def.GetRecSize() );
	UpdateKeySize( pDoc->Def.GetKeySize() );
	UpdateData(FALSE);
}

//////////////////////////////////////////////////////////////////
////////////////////////////////////////  Manage the Field list

/////////////////////////////////////////////////////////////////////
// Change field names in sort and output field, if they exist there,
// Keeps Sort and Output synched with Field list
void CSrtView::ChangeInLists(const char * OldName, const char * NewName)
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	int i;

	i = pDoc->Def.FindSort( OldName );
	if( i != -1 )
	{
		pDoc->Def.SetSortName( i, NewName );
		CString Descr = pDoc->Def.GetSortDescr(i);
		m_SortList.DeleteString( i );
		m_SortList.InsertString( i, Descr );
	}

	i = pDoc->Def.FindOutput( OldName );
	if( i != -1 )
	{
		pDoc->Def.SetOutputName( i, NewName );
		CString Descr = pDoc->Def.GetOutputDescr(i);
		m_OutList.DeleteString( i );
		m_OutList.InsertString( i, Descr );
	}
}

/////////////////////////////////////////////////////////////////////
// Remove field names in sort and output field, if they exist there,
// Keeps Sort and Output synched with Field list
void CSrtView::RemoveFromLists(const char * Name)
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	int i;

	i = pDoc->Def.FindSort( Name );
	if( i != -1 )
	{
		pDoc->Def.RemoveSort( i );
		m_SortList.DeleteString( i );
	}

	i = pDoc->Def.FindOutput( Name );
	if( i != -1 )
	{
		pDoc->Def.RemoveOutput( i );
		m_OutList.DeleteString( i );
	}
}

void CSrtView::OnDblclkFldlist() 
{
	FldEdit Tmp;
	CString OldName;
	int i;

	i = m_FldList.GetCurSel();
	if( i != LB_ERR )
	{
		CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		Tmp.m_Name = pDoc->Def.GetFieldName(i);
		OldName = Tmp.m_Name;
		Tmp.m_Size = pDoc->Def.GetFieldSize(i);
		Tmp.m_Offset = pDoc->Def.GetFieldOffset(i);
		Tmp.m_Type = pDoc->Def.GetFieldType(i);
		if( Tmp.DoModal() == IDOK )
		{	
			CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
			CString Descr;
			if( Tmp.m_Name.CompareNoCase( OldName ) != 0 )
				ChangeInLists( OldName, Tmp.m_Name );
			pDoc->Def.SetField( Tmp.m_Name, Tmp.m_Type, (WORD)Tmp.m_Size, (WORD)Tmp.m_Offset, i );
			Descr = pDoc->Def.GetFieldDescr(i);
			m_FldList.DeleteString( i );
			m_FldList.InsertString( i, Descr );
			m_FldList.SetCurSel(i);
			UpdateRecSize( pDoc->Def.GetRecSize() );
			UpdateKeySize( pDoc->Def.GetKeySize() );
			pDoc->SetModifiedFlag();
		}
	}
}

void CSrtView::OnFldadd() 
{
	FldEdit Tmp;

	if( Tmp.DoModal() == IDOK )
	{	
		CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		CString Descr;
		pDoc->Def.AddField( Tmp.m_Name, Tmp.m_Type, (WORD)Tmp.m_Size, (WORD)Tmp.m_Offset );
		Descr = pDoc->Def.GetFieldDescr(pDoc->Def.GetFieldCount()-1);
		m_FldList.AddString( Descr );
		UpdateRecSize( pDoc->Def.GetRecSize() );
		pDoc->SetModifiedFlag();
	}
}

void CSrtView::OnFlddel() 
{
	int i;

	i = m_FldList.GetCurSel();
	if( i!=LB_ERR )
	{
		CSrtDoc * pDoc = (CSrtDoc*)GetDocument();

		RemoveFromLists( pDoc->Def.GetFieldName(i) );
		pDoc->Def.RemoveField(i);
		m_FldList.DeleteString( i );
		UpdateRecSize( pDoc->Def.GetRecSize() );
		UpdateKeySize( pDoc->Def.GetKeySize() );
		pDoc->SetModifiedFlag();
		if( m_FldList.GetCount()==0 )
			return;
		m_FldList.SetCurSel( min( m_FldList.GetCount()-1, i ) );
	}
}

void CSrtView::OnFldup() 
{
	int i;
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		
	i = m_FldList.GetCurSel();
	if( i!=LB_ERR && i )
	{
		pDoc->Def.MoveFieldUp( i );
		MoveLine( m_FldList, i, i-1 );
		pDoc->SetModifiedFlag();
	}
}

void CSrtView::OnFlddwn() 
{
	int i;
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		
	i = m_FldList.GetCurSel();
	if( i!=LB_ERR && i+1 != pDoc->Def.GetFieldCount() )
	{
		pDoc->Def.MoveFieldDown( i );
		MoveLine( m_FldList, i, i+1 );
		pDoc->SetModifiedFlag();
	}
}

//////////////////////////////////////////////////////////////////
////////////////////////////////////////  Manage the Sort By list
void CSrtView::OnDblclkSrtlist() 
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	SrtEdit Tmp;
	int i,j;

	i = m_SortList.GetCurSel();
	if( i != LB_ERR )
	{
		for( j=0; j< pDoc->Def.GetFieldCount(); j++ )
			Tmp.FieldList.Add( pDoc->Def.GetFieldName(j) );

		Tmp.m_Field=pDoc->Def.GetSortName(i);
		Tmp.m_Sensitive=pDoc->Def.IsSensitive(i);
		if( Tmp.DoModal() == IDOK )
		{	
			CString Descr;
			pDoc->Def.SetSort( Tmp.m_Field, Tmp.m_Sensitive, i);
			Descr = pDoc->Def.GetSortDescr(i);
			m_SortList.DeleteString( i );
			m_SortList.InsertString( i, Descr );
			m_SortList.SetCurSel(i);

			UpdateKeySize( pDoc->Def.GetKeySize() );
		}
	}
}	
	
void CSrtView::OnSrtadd() 
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	SrtEdit Tmp;
	int i;

	for( i=0; i< pDoc->Def.GetFieldCount(); i++ )
		Tmp.FieldList.Add( pDoc->Def.GetFieldName(i) );

	if( Tmp.DoModal() == IDOK )
	{	
		CString Descr;
		pDoc->Def.AddSort( Tmp.m_Field, Tmp.m_Sensitive);
		Descr = pDoc->Def.GetSortDescr(pDoc->Def.GetSortCount()-1);
		m_SortList.AddString( Descr );
		UpdateKeySize( pDoc->Def.GetKeySize() );
		pDoc->SetModifiedFlag();
	}
}

void CSrtView::OnSrtdel() 
{
	int i;
	i = m_SortList.GetCurSel();
	if( i!=LB_ERR )
	{
		CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		pDoc->Def.RemoveSort(i);
		m_SortList.DeleteString( i );		
		UpdateKeySize( pDoc->Def.GetKeySize() );
		pDoc->SetModifiedFlag();
	}
	if( m_SortList.GetCount()==0 )
		return;
	m_SortList.SetCurSel( min( m_SortList.GetCount()-1, i ) );
	
}

void CSrtView::OnSrtdwn() 
{
	int i;
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		
	i = m_SortList.GetCurSel();
	if( i!=LB_ERR && i+1 != pDoc->Def.GetSortCount() )
	{
		pDoc->Def.MoveSortDown( i );
		MoveLine( m_SortList, i, i+1 );
		pDoc->SetModifiedFlag();
	}
	
}

void CSrtView::OnSrtup() 
{
	int i;
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		
	i = m_SortList.GetCurSel();
	if( i!=LB_ERR && i )
	{
		pDoc->Def.MoveSortUp( i );
		MoveLine( m_SortList, i, i-1 );
		pDoc->SetModifiedFlag();
	}

}

//////////////////////////////////////////////////////////////////
////////////////////////////////////////  Manage the Output list
void CSrtView::OnDblclkOutlist() 
{
	OutEdit Tmp;
	int i,j;

	i = m_OutList.GetCurSel();
	if( i != LB_ERR )
	{
		CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		
		for( j=0; j< pDoc->Def.GetFieldCount(); j++ )
			Tmp.FieldList.Add( pDoc->Def.GetFieldName(j) );

		Tmp.m_Field = pDoc->Def.GetOutputName(i);
		Tmp.m_Postfix = pDoc->Def.GetOutputPostfix(i);

		Tmp.m_NoSep = !pDoc->Def.HasSep(i);
		Tmp.m_PermitBlank = pDoc->Def.AllowBlank(i);
		Tmp.m_EndLine = pDoc->Def.IsEndLine(i);
		Tmp.m_NoTrim = !pDoc->Def.IsTrim(i);
	
		if( Tmp.DoModal() == IDOK )
		{	
			CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
			CString Descr;
			WORD Options=0;

			Options = (Tmp.m_NoSep?MOutput::OO_NOSEP:0) | 
				(Tmp.m_PermitBlank?MOutput::OO_ALLOWBLANK:0)|
				(Tmp.m_EndLine?MOutput::OO_ENDLINE:0) |
				(Tmp.m_NoTrim?MOutput::OO_NOTRIM:0);
			
			pDoc->Def.SetOutput( Tmp.m_Field, Tmp.m_Postfix, Options, i );
			Descr = pDoc->Def.GetOutputDescr(i);
			m_OutList.DeleteString( i );
			m_OutList.InsertString( i, Descr );
			m_OutList.SetCurSel(i);
			pDoc->SetModifiedFlag();
		}
	}
}

void CSrtView::OnOutadd() 
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	OutEdit Tmp;
	int i;

	for( i=0; i< pDoc->Def.GetFieldCount(); i++ )
		Tmp.FieldList.Add( pDoc->Def.GetFieldName(i) );

	if( Tmp.DoModal() == IDOK )
	{
		CString Descr;
		WORD Options=0;
		Options =	(Tmp.m_NoSep?MOutput::OO_NOSEP:0) |
			(Tmp.m_PermitBlank?MOutput::OO_ALLOWBLANK:0)|
			(Tmp.m_EndLine?MOutput::OO_ENDLINE:0) |
			(Tmp.m_NoTrim?MOutput::OO_NOTRIM:0);

		pDoc->Def.AddOutput( Tmp.m_Field, Tmp.m_Postfix, Options);
		Descr = pDoc->Def.GetOutputDescr(pDoc->Def.GetOutputCount()-1);
		m_OutList.AddString( Descr );
		pDoc->SetModifiedFlag();
	}
	
}

void CSrtView::OnOutdel() 
{
	int i;
	i = m_OutList.GetCurSel();
	if( i!=LB_ERR )
	{
		CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		pDoc->Def.RemoveOutput(i);
		m_OutList.DeleteString( i );
		pDoc->SetModifiedFlag();
		if( m_OutList.GetCount()==0 )
			return;
		m_OutList.SetCurSel( min( m_OutList.GetCount()-1, i ) );
	}
	
}

void CSrtView::OnOutdwn() 
{
	int i;
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		
	i = m_OutList.GetCurSel();
	if( i!=LB_ERR && i+1 != pDoc->Def.GetOutputCount() )
	{
		pDoc->Def.MoveOutputDown( i );
		MoveLine( m_OutList, i, i+1 );
		pDoc->SetModifiedFlag();
	}
		
}

void CSrtView::OnOutup() 
{
	int i;
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
		
	i = m_OutList.GetCurSel();
	if( i!=LB_ERR && i )
	{
		pDoc->Def.MoveOutputUp( i );
		MoveLine( m_OutList, i, i-1 );
		pDoc->SetModifiedFlag();
	}

}

//////////////////////////////////////////////////////////////////
////////////////////////////////////////  Misc control interaction

//////////////////////////////////////////////////////////////////
//  OnRun - Performs simple validation, then starts the sort.
void CSrtView::OnRun() 
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	CString Result;

	UpdateData( TRUE );

	if (m_InputFile=="")
	{
		MessageBox( "Please specify input filename", "Error" );
		return;
	}
	if (m_OutputFile=="")
	{
		MessageBox( "Please specify output filename", "Error" );
		return;
	}
	if( pDoc->Def.GetFieldCount()==0 )
	{
		MessageBox( "Please specify field layout", "Error" );
		return;
	}
	if( pDoc->Def.GetOutputCount()==0 )
	{
		MessageBox( "Please specify output format", "Error" );
		return;
	}
	// Sort isn't important, incase they just want to output in same order

	pDoc->Def.SetAscending( !m_Descending );
	pDoc->Def.SetRecSep( m_HasSep );
	if( pDoc->Def.DoSort( &m_Status, Result ) == FALSE )
		MessageBox( Result, pDoc->Def.DoSort( &m_Status, Result )?"Information":"Error" );
}


//////////////////////////////////////////////////////////////////
//  OnPickinp - Presents user with file-open dialog box.
void CSrtView::OnPickinp() 
{

	CFileDialog Tmp(TRUE);

	if( Tmp.DoModal()==IDOK )
	{
		m_InputFile=Tmp.GetPathName();
		UpdateData(FALSE);
		OnChangeFile();
	}

}

//////////////////////////////////////////////////////////////////
//  OnPickout - Presents user with file-save dialog box.
void CSrtView::OnPickout() 
{

	CFileDialog Tmp(FALSE);

	if( Tmp.DoModal()==IDOK )
	{
		m_OutputFile=Tmp.GetPathName();
		UpdateData(FALSE);
		OnChangeFile();
	}

}

void CSrtView::OnChangeFile()
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	UpdateData( TRUE );
	pDoc->Def.SetOutputFile( m_OutputFile );
	pDoc->Def.SetFile( m_InputFile );
}

void CSrtView::OnCalcOff()	/// Calculate Offsets
{
	int i;
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	pDoc->Def.CalcOffsets();

	if( m_FldList.GetCount() )
		m_FldList.ResetContent();
	for( i=0; i < pDoc->Def.GetFieldCount(); i++ )
		m_FldList.AddString( pDoc->Def.GetFieldDescr(i) );
}


void CSrtView::OnHassep() 
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	UpdateData(TRUE);
	pDoc->Def.SetRecSep( m_HasSep );
}

void CSrtView::OnDescend() 
{
	CSrtDoc * pDoc = (CSrtDoc*)GetDocument();
	UpdateData(TRUE);
	pDoc->Def.SetAscending( !m_Descending );
}


void CSrtView::UpdateRecSize( int RecSize )
{
	char Tmp[6];
	wsprintf( Tmp, "%u", RecSize );
	m_RecSize.SetWindowText(Tmp);
}

void CSrtView::UpdateKeySize( int KeySize )
{
	char Tmp[6];
	wsprintf( Tmp, "%u", KeySize );
	m_KeySize.SetWindowText(Tmp);
}

void CSrtView::MoveLine( CListBox & List, int OldPos, int NewPos )
{
	CString X;
	List.GetText( OldPos, X );
	List.DeleteString( OldPos );
	List.InsertString( NewPos, X );
	List.SetCurSel( NewPos );
}

// OnInitialUpdate - Here we resize the view to fit the dialog, for appearance
void CSrtView::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit(FALSE);
}

