 
class MField : public CObject
{
public:
	// For MFC serialization
	DECLARE_SERIAL( MField );
	MField( ) {DataSize=0;Offset=0; Status=FS_NULL;} ;
	void Serialize( CArchive & ar );
	
	virtual ~MField() {};
	MField( const char *xName, WORD xSize, WORD xOffset=-1 ) { Name = xName; DataSize=xSize; Offset=xOffset; };
	MField( MField const & Src );

	// Operations    
	virtual CString GetAsString() { return( Data ); };
	virtual void SetAsString( const char * Source );
	virtual void SetFromBuffer( const char * Buffer );
	virtual void GetToBuffer( CString Buffer );
	virtual MField operator=(char* Src) { SetAsString(Src); return(*this);}
	virtual int GetAsInt() {return( atoi( Data ) );};
	virtual void SetAsInt(int Val);
	

	// Attributes
	virtual char * GetType() { return( "MField" ); };
	void SetName( char * strName ) { Name = strName; };
	CString GetName() { return( Name ); };    
	WORD GetSize() { return( DataSize ); };
	WORD GetOffset() { return( Offset ); };
	void SetOffset( WORD xOffset ) { Offset=xOffset; }
	BOOL IsName( const char * Src ) { return( !stricmp( Src, Name ) ); };

	BOOL IsNull() { return( Status&FS_NULL ); };
	virtual BOOL IsValid() { return( !(Status&FS_NULL) ); };
	void SetNull() { Data=""; Status |= FS_NULL; };
	
	BOOL WillTrunc() { return( Status&FS_TRUNC ); };
	void SetTrunc( BOOL Trunc ) { Status=(BYTE) (Trunc?Status&(~FS_TRUNC):Status|FS_TRUNC);};

	enum States { FS_NULL=1, FS_TRUNC=2 };

protected:
	void ClearNull() { Status &= (~FS_NULL); };

	CString Name, Data;
	WORD Offset, DataSize;
	BYTE Status;

};

//////////////////////////////////////////////////////////////////////
// The F_String is a field type derived from MField.  It is the only
// field type currently available for the this program.
class F_String : public MField
{
public:
	DECLARE_SERIAL( F_String );
	F_String() { AutoTrim=0; }; // For Serialization requirements.
	virtual ~F_String() {};
	F_String( const char *xName, WORD xSize, WORD xOffset=-1 ) : MField(xName,xSize,xOffset) {AutoTrim=0;};
	void Serialize( CArchive & ar );
	virtual void SetAsString( const char * Src );
	virtual void SetFromBuffer( const char * Buffer );
	virtual void SetAsInt( int Val);
	virtual char * GetType() { return( "string" ); };

	BOOL GetAutoTrim() { return( !!AutoTrim ); };
	void SetAutoTrim( int Mode ) { AutoTrim=Mode?'\0':' '; };

protected:
	void FixIt( CString * Str);
	BYTE AutoTrim;
};

/////////////////////////////////////////////////////////////////////
//  The FieldException class handles exceptions for the MField objects
#define ThrowFieldException(X,Y) throw new FieldException(X,Y,__LINE__,__FILE__)

class FieldException : public CException
{
public:
	DECLARE_DYNAMIC(FieldException)
public:
	FieldException( char *Msg, const char *Field, int Line, char * File );
	FieldException( FieldException  & X );
	virtual BOOL GetErrorMessage( LPTSTR lpszError, UINT nMaxError, PUINT pnHelpContext = NULL );
private:
	CString Msg;
};


