#include "stdafx.h"

#include "sortdef.h"

IMPLEMENT_SERIAL( MField, CObject, 1 )


// Set field value from string
void MField::SetAsString( const char * Src)
{
	if(  !WillTrunc() && strlen(Src) >= DataSize )
		ThrowFieldException( "Data size error", Name );
	Data = Src;
	ClearNull();
}

//Set field value from buffer.  Uses Offset
void MField::SetFromBuffer( const char * Src)
{
	if(  Offset==-1 || DataSize==0 )
		ThrowFieldException( "Offset/Size error", Name );
	Data = CString(Src+Offset, DataSize);
	ClearNull();
}

// Set field value from an integer
void MField::SetAsInt( int Val )
{
	char Tmp[16];
	wsprintf(Tmp, "%d", Val );
	SetAsString( Tmp );
}

// Copy constructor
MField::MField( MField const & Src )
{
	Name = Src.Name;
	Data = Src.Data;
	Offset = Src.Offset;
	DataSize = Src.DataSize;
	Status = Src.Status;
}

// Sets a buffer's space with data from the field
void MField::GetToBuffer( CString Buffer )
{
	if( Offset!=-1 )
	{
		for( int i=0; i< DataSize; i++ )
			Buffer.SetAt(i+Offset, Data[i]);
	}
	else
		ThrowFieldException( "Invalid Offset", Name );
}


// MFC-style Serialization support
void MField::Serialize( CArchive & ar )
{
	CObject::Serialize( ar );
	// Note: Since this serialize is intended to maintain field
	// attirubtes, and not field data, we *do not* save and retrieve
	// the Data and Status members.
	if( ar.IsStoring() )
	{
		ar << Name;
		ar.Write(&Offset, sizeof(Offset));
		ar.Write(&DataSize, sizeof(DataSize));
	}
	else
	{
		ar >> Name;
		ar.Read(&Offset, sizeof(Offset));
		ar.Read(&DataSize, sizeof(DataSize));
		SetNull();
	}
}

	

///////////////////////////////////////////////////////////////
// F_String class

IMPLEMENT_SERIAL( F_String, MField, 1 );

// Handles the trim/no-trim coding for strings
void F_String::FixIt( CString * Str)
{
	if( GetAutoTrim() )
		Str->TrimRight();
	else
	while( strlen(*Str) < DataSize )
			*Str += " ";

}

void F_String::SetAsString( const char *Src )
{
	CString Tmp=Src;
	FixIt( &Tmp );
	MField::SetAsString(Tmp);
	
}

void F_String::SetFromBuffer( const char * Buffer )
{
	MField::SetFromBuffer( Buffer );
	FixIt(&Data);
}

void F_String::SetAsInt( int Val)
{
	char Tmp[16];
	wsprintf( Tmp, "%d", Val );
	SetAsString(Tmp);
}

void F_String::Serialize( CArchive & ar )
{
	MField::Serialize( ar );
	if( ar.IsStoring() )
		ar.Write(&AutoTrim, sizeof(AutoTrim));
	else
		ar.Read( &AutoTrim, sizeof(AutoTrim));
}

//////////////////////////////////////////////////////////////////
// FieldException class
//
// This class is thrown by the MField and derived classes upon error
FieldException::FieldException( char *xMsg, const char *Field, int Line, char * File )
{
	char Tmp[64];
	
	wsprintf( Tmp, "(%s:%u)", File, Line );

	Msg = xMsg;
	Msg += " ";
	Msg += Field;
	Msg += " ";
	Msg += Tmp;
	
}

IMPLEMENT_DYNAMIC( FieldException,CException )

FieldException::FieldException( FieldException  & X )
{
	Msg = X.Msg;
}
BOOL FieldException::GetErrorMessage( LPTSTR lpszError, UINT nMaxError, PUINT pnHelpContext )
{
	if( pnHelpContext )
		*pnHelpContext=0;
	strncpy( lpszError, Msg, min( nMaxError, strlen(Msg)) );
	lpszError[min( nMaxError, strlen(Msg))]='\0';
	return( TRUE );

}


