
#include "mfield.h"
#include "msort.h"
#include "mflat.h"
#include "moutput.h"
#include "msortstr.h"

/////////////////////////////////////////////////////////////////////
//  The MSortDef class, derived from the MFlatFile class, adds the
// ability to contain and manage Sort and Output field definitions.
// It is also able to perform a sort.
class MSortDef : public MFlatFile
{
public:
	DECLARE_SERIAL( MSortDef )

	MSortDef() { KeySize=0; RecSep=FALSE;}
	~MSortDef() { Reset(); };
	void Serialize( CArchive & ar );   

// General support
	void Reset();
	BOOL DoSort( CWnd * ToUpdate, CString & Result );
	void AddRecordSort(WORD Rec);
	void SetAscending( int Mode ) { Ascending=Mode; }
	BOOL GetAscending() { return( Ascending );}
	BOOL HasRecSep() { return( RecSep ); }
	void SetRecSep( int Mode ) { RecSep = !!Mode; };
	int FindSort( const char *Name );
	int FindOutput( const char * Name );

// Sort Support
	void AddSort( CString Name, BOOL Sensitive );
	void InsertSort(CString Name, BOOL Sensitive, int i );
	void SetSort( CString Name, BOOL Sensitive, int i );
	void RemoveSort( int i );

	// High-level functions
	int GetSortCount() { return(SortArray.GetSize()); }
	WORD GetKeySize() { return( KeySize ); }
	CString GetSortDescr( int i );
	void MoveSortUp( int i ) { if( i ) Move( SortArray, i, i-1 ); }
	void MoveSortDown( int i ) { if( i < GetSortCount() ) Move(SortArray, i,i+1); }
	
	// Lowlevel functions
	MSort* GetSort(int i) { return(((MSort*)SortArray[i])); }
	CString GetSortName( int i ) { return( GetSort(i)->GetFieldName());}
	void SetSortName( int i , const char *S) { GetSort(i)->SetFieldName(S);}
	BYTE IsSensitive( int i ) { return( GetSort(i)->IsSensitive() ); }
	WORD GetSortedRec(int i) { return((WORD) atoi( ((const char*)Sorted.GetAt(Sorted.FindIndex(i)))+KeySize ) ); }

// File support
	void SetOutputFile( const char * xFile ) { OutputName = xFile; };
	void CloseOutputFile();
	BOOL CreateOutputFile( );

// Output support

	// Hi-level functions
	void AddOutput( CString Name, CString Postfix, WORD Options );
	void InsertOutput( CString Name, CString Postfix, WORD Options, int i );
	void SetOutput( CString Name, CString Postfix, WORD Options, int i );
	void RemoveOutput( int i ) { delete GetOutput(i); OutputArray.RemoveAt(i); }
	void MoveOutputUp( int i ) { if( i ) Move( OutputArray, i, i-1 ); }
	void MoveOutputDown( int i ) { if( i < GetSortCount() ) Move(OutputArray, i,i+1); }

	// Low-level
	MOutput * GetOutput(int i) { return( (MOutput*)OutputArray[i] ); }
	int GetOutputCount() { return( OutputArray.GetSize()); }
	CString GetOutputName( int i ) { return( ((MOutput*)OutputArray[i])->GetFieldName() );}
	void SetOutputName( int i, const char * S ) { ((MOutput*)OutputArray[i])->SetFieldName(S) ;}

	CString GetOutputPostfix( int i ) { return( ((MOutput*)OutputArray[i])->GetPostfix() );}
	CString GetOutputDescr( int i );
	const char * GetOutputFile() { return( OutputName );}
	void OutputRecord();
	
	BOOL IsEndLine(int i) { return( GetOutput(i)->IsEndLine() );}
	BOOL IsTrim(int i) { return( GetOutput(i)->IsTrim()  );}
	BOOL HasSep(int i) { return( GetOutput(i)->HasSep() );}

	BOOL AllowBlank(int i) { return( GetOutput(i)->AllowBlank() );}
	
	
protected:
	CObArray SortArray;
	WORD KeySize;

	CObArray OutputArray;
	MSortStr Sorted;
	
	CString OutputName;
	CFile Output;
	
	BOOL RecSep, Ascending;
};

